@@ -1,7 +1,7 @@
 /*
- * tnef.c -- extract files from microsoft TNEF format
+ * main.c -- extract files from microsoft TNEF format
  *
- * Copyright (C)1999, 2000, 2001 Mark Simpson <damned@world.std.com>
+ * Copyright (C)1999-2003 Mark Simpson <damned@world.std.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,19 +25,43 @@
 #  include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <assert.h>
 #include <stdio.h>
+
+#if STDC_HEADERS
+#  include <stdlib.h>
+#  include <string.h>
+#else
+extern int strcmp (consr char *, const char *);
+extern void abort (void);
+extern void exit (int);
+#endif
+
+#ifndef _
+/* This is for other GNU distributions with internationalized messages.
+   When compiling libc, the _ macro is predefined.  */
+# ifdef HAVE_LIBINTL_H
+#  include <libintl.h>
+#  define _(msgid)	gettext (msgid)
+# else
+#  define _(msgid)	(msgid)
+# endif
+#endif
+
 #if HAVE_GETOPT_LONG
 #  include <getopt.h>
 #else
 #  include "getopt_long.h"
 #endif /* HAVE_GETOPT_LONG */
 
+#include "alloc.h"
+
 #include "tnef.h"
 
 /* COPYRIGHTS & NO_WARRANTY -- defined to make code below a little nicer to
    read */
 static const char* COPYRIGHTS = \
-"Copyright (C) 1999 by Mark Simpson\n"
+"Copyright (C) 1999-2003 by Mark Simpson\n"
 "Copyright (C) 1997 by Thomas Boll (original code)";
 static const char* NO_WARRANTY = \
 "%s comes with ABSOLUTELY NO WARRANTY.\n"
@@ -45,33 +69,32 @@ static const char* NO_WARRANTY = \
 "Public License.  For more information about these matters, see the file\n"
 "named COPYING.";
 static const char* USAGE = \
-"-f FILE, --file=FILE\tuse FILE as input ('-' == stdin)\n"
-"-C DIR, --directory=DIR\tunpack files into DIR\n"
-"-t,     --list      \tlist files, do not extract\n"
-"-w,     --interactive\task for confirmation for every action\n"
-"        --confirmation\tsame as -w\n"
-"        --overwrite \tOverwrite existing files\n"
-"        --number-backups\tIf need to overwrite file FOO, make FOO.1 instead\n"
-"        --use-paths \tUse pathnames for files if found in the TNEF file\n"
-"                    \t(for security reasons paths to included files are\n"
-"                    \t ignored by default)\n"
-"-h,     --help      \tshow this message\n"
-"-V,     --version   \tdisplay version and copyright\n"
-"-v,     --verbose   \tproduce verbose output\n"
-"        --debug     \tproduce a lot of output\n"
+"-f FILE,--file=FILE     \tuse FILE as input ('-' == stdin)\n"
+"-C DIR, --directory=DIR \tunpack files into DIR\n"
+"-x SIZE --maxsize=SIZE  \tlimit maximum size of extracted archive (bytes)\n"
+"-t,     --list          \tlist files, do not extract\n"
+"-w,     --interactive   \task for confirmation for every action\n"
+"        --confirmation  \tsame as -w\n"
+"        --overwrite     \tOverwrite existing files\n"
+"        --number-backups\tInstead of overwriting file FOO,\n"
+"                        \t  create FOO.n instead\n"
+"        --use-paths     \tUse pathnames for files if found in the TNEF\n" 
+"                        \t  file (for security reasons paths to included\n"
+"                        \t  files are ignored by default)\n"
+"-h,     --help          \tshow this message\n"
+"-V,     --version       \tdisplay version and copyright\n"
+"-v,     --verbose       \tproduce verbose output\n"
+"        --debug     	 \tproduce a lot of output\n"
 "\n"
-"-d                  \t[same as -C; deprecated]\n"
-"-l                  \t[same as -t; deprecated]\n"
-"-n, --dry_run       \t[same as -t; deprecated]\n"
 "\nIf FILE is not specified standard input is used\n"
-"\nReport bugs to <damned@world.std.com>\n";
+"\nReport bugs to <%s>\n";
 
 
 static void
 usage (char* prog)
 {
     fprintf (stdout, "%s: [options] [FILE]\n", prog);
-    fprintf (stdout, USAGE);
+    fprintf (stdout, USAGE, PACKAGE_BUGREPORT);
 }
 
 
@@ -79,29 +102,30 @@ static void
 parse_cmdline (int argc, char **argv,
                char **in_file,
                char **out_dir,
+               size_t *max_size,
                int *flags)
 {
     int i = 0;
     int option_index = 0;
     static struct option long_options[] = 
     { 
-        {"file", required_argument, 0, 'f' },
-        {"directory", required_argument, 0, 'C' },
-        {"help", no_argument, 0, 'h'},
-        {"verbose", no_argument, 0, 'v'},
-        {"dry_run", no_argument, 0, 'l'},
-        {"list", no_argument, 0, 'l'}, /* for now same as -n */
         {"confirmation", no_argument, 0, 'w' },
-        {"interactive", no_argument, 0, 'w' },
-        {"version", no_argument, 0, 'V'},
-        {"use-paths", no_argument, 0, 0},
         {"debug", no_argument, 0, 0},
-        {"overwrite", no_argument, 0, 0 },
+        {"directory", required_argument, 0, 'C' },
+        {"file", required_argument, 0, 'f' },
+        {"help", no_argument, 0, 'h'},
+        {"interactive", no_argument, 0, 'w' },
+        {"list", no_argument, 0, 't'}, /* for now same as -n */
+        {"maxsize", required_argument, 0, 'x' },
         {"number-backups", no_argument, 0, 0 },
+        {"overwrite", no_argument, 0, 0 },
+        {"use-paths", no_argument, 0, 0},
+        {"verbose", no_argument, 0, 'v'},
+        {"version", no_argument, 0, 'V'},
         { 0, 0, 0, 0 }
     };
 
-    while ((i = getopt_long (argc, argv, "f:C:d:vVwhtnl", 
+    while ((i = getopt_long (argc, argv, "f:C:d:x:vVwht",
                              long_options, &option_index)) != -1)
     {
         switch (i) 
@@ -134,7 +158,7 @@ parse_cmdline (int argc, char **argv,
             break;
 
         case 'V':
-            fprintf (stderr, "%s %s\n", PACKAGE, VERSION);
+            fprintf (stderr, "%s\n", PACKAGE_STRING);
             fprintf (stderr, COPYRIGHTS);
             fprintf (stderr, "\n");
             fprintf (stderr, NO_WARRANTY, PACKAGE, PACKAGE);
@@ -156,17 +180,26 @@ parse_cmdline (int argc, char **argv,
             (*out_dir) = optarg;
             break;
 
+        case 'x':
+            {
+                char *end_ptr = NULL;
+                (*max_size) = strtoul (optarg, &end_ptr, 10);
+                if (*end_ptr != '\0')
+                {
+                    fprintf (stderr, 
+                             "Invalid argument to --maxsize/-x option: '%s'\n",
+                             optarg);
+                    exit (-1);
+                }
+            }
+            break;
+
         case 'h':
             usage(argv[0]);
             exit (0);
             break;
 
-                                /* for now -t -l -n --list and --dry-run
-                                   are the same.  -l,-n,--dry-run are all
-                                   deprecated.  */
         case 't':
-        case 'l':
-        case 'n':
             *flags |= LIST;
             break;
 
@@ -204,8 +237,16 @@ main (int argc, char *argv[])
     char *in_file = NULL;
     char *out_dir = NULL;
     int flags = NONE;
+    size_t max_size = 0;
     
-    parse_cmdline (argc, argv, &in_file, &out_dir, &flags);
+    parse_cmdline (argc, argv, &in_file, &out_dir, &max_size, &flags);
+
+    set_alloc_limit (max_size);
+    if (flags & DBG_OUT)
+    {
+        fprintf (stdout, "setting alloc_limit to: %lu\n", 
+                 (unsigned long)max_size);
+    }
 
     /* open the file */
     if (in_file)
